Package: ClusterR
Type: Package
Title: Gaussian Mixture Models, K-Means, Mini-Batch-Kmeans, K-Medoids
        and Affinity Propagation Clustering
Version: 1.2.2
Date: 2020-05-12
Authors@R: c( person("Lampros", "Mouselimis", email = "mouselimislampros@gmail.com", role = c("aut", "cre")), person("Conrad", "Sanderson", role = "cph", comment = "Author of the C++ Armadillo library"), person("Ryan", "Curtin", role = "cph", comment = "Author of the C++ Armadillo library"), person("Siddharth", "Agrawal", role = "cph", comment = "Author of the C code of the Mini-Batch-Kmeans algorithm (https://github.com/siddharth-agrawal/Mini-Batch-K-Means)"), person("Brendan", "Frey", email = "frey@psi.toronto.edu", role = "cph", comment = "Author of the matlab code of the Affinity propagation algorithm (for commercial use please contact the author of the matlab code)"), person("Delbert", "Dueck", role = "cph", comment = "Author of the matlab code of the Affinity propagation algorithm") )  
Maintainer: Lampros Mouselimis <mouselimislampros@gmail.com>
BugReports: https://github.com/mlampros/ClusterR/issues
URL: https://github.com/mlampros/ClusterR
Description: Gaussian mixture models, k-means, mini-batch-kmeans, k-medoids and affinity propagation clustering with the option to plot, validate, predict (new data) and estimate the optimal number of clusters. The package takes advantage of 'RcppArmadillo' to speed up the computationally intensive parts of the functions. For more information, see (i) "Clustering in an Object-Oriented Environment" by Anja Struyf, Mia Hubert, Peter Rousseeuw (1997), Journal of Statistical Software, <doi:10.18637/jss.v001.i04>; (ii) "Web-scale k-means clustering" by D. Sculley (2010), ACM Digital Library, <doi:10.1145/1772690.1772862>; (iii) "Armadillo: a template-based C++ library for linear algebra" by Sanderson et al (2016), The Journal of Open Source Software, <doi:10.21105/joss.00026>; (iv) "Clustering by Passing Messages Between Data Points" by Brendan J. Frey and Delbert Dueck, Science 16 Feb 2007: Vol. 315, Issue 5814, pp. 972-976, <doi:10.1126/science.1136800>.
License: GPL-3
Encoding: UTF-8
SystemRequirements: libarmadillo: apt-get install -y libarmadillo-dev
        (deb), libblas: apt-get install -y libblas-dev (deb),
        liblapack: apt-get install -y liblapack-dev (deb),
        libarpack++2: apt-get install -y libarpack++2-dev (deb),
        gfortran: apt-get install -y gfortran (deb), libgmp3: apt-get
        install -y libgmp3-dev (deb), libfftw3: apt-get install -y
        libfftw3-dev (deb), libtiff5: apt-get install -y libtiff5-dev
        (deb)
LazyData: TRUE
Depends: R(>= 3.2), gtools
Imports: Rcpp (>= 0.12.5), graphics, grDevices, utils, stats, gmp,
        ggplot2
LinkingTo: Rcpp, RcppArmadillo (>= 0.9.1)
Suggests: OpenImageR, FD, testthat, covr, knitr, rmarkdown
VignetteBuilder: knitr
RoxygenNote: 7.1.0
NeedsCompilation: yes
Packaged: 2020-05-12 08:25:58 UTC; lampros
Author: Lampros Mouselimis [aut, cre],
  Conrad Sanderson [cph] (Author of the C++ Armadillo library),
  Ryan Curtin [cph] (Author of the C++ Armadillo library),
  Siddharth Agrawal [cph] (Author of the C code of the Mini-Batch-Kmeans
    algorithm
    (https://github.com/siddharth-agrawal/Mini-Batch-K-Means)),
  Brendan Frey [cph] (Author of the matlab code of the Affinity
    propagation algorithm (for commercial use please contact the author
    of the matlab code)),
  Delbert Dueck [cph] (Author of the matlab code of the Affinity
    propagation algorithm)
Repository: CRAN
Date/Publication: 2020-05-12 09:10:03 UTC
Built: R 4.0.0; x86_64-w64-mingw32; 2020-05-12 20:59:11 UTC; windows
Archs: x64
